@@ -77,6 +77,8 @@ gint64 get_disc_size_dvdr   (int fd, int mmc_profile);
 int    get_read_write_speed (int fd, int *read_speed, int *write_speed);
 int    get_disc_status      (int fd, int *empty, int *is_rewritable, int *is_blank);
 
+#define ENABLE_NON_HAL_FALLBACK_CODE 1
+
 #ifndef INVALID_HANDLE
 #define INVALID_HANDLE (GINT_TO_POINTER(-1))
 #endif
@@ -206,22 +208,6 @@ close_ioctl_handle (gpointer handle)
 
 #endif
 
-static gboolean
-use_gnome_mount (void)
-{
-#ifndef USE_HAL
-	return FALSE;
-#endif
-
-	char *tmp = g_find_program_in_path ("gnome-mount");
-	if (tmp != NULL) {
-		g_free (tmp);
-		return TRUE;
-	} else {
-		return FALSE;
-	}
-}
-
 #ifdef USE_HAL
 static LibHalContext *
 get_hal_context (void)
@@ -991,7 +977,7 @@ hal_drive_get_first_child_udi (NautilusBurnDrive *drive)
 	}
 
 	if (num_devices <= 0) {
-		g_warning ("No HAL devices found for UDI '%s'", drive->priv->udi);
+		/*g_warning ("No HAL devices found for UDI '%s'", drive->priv->udi);*/
 
 		goto done;
 	}
@@ -1102,11 +1088,15 @@ nautilus_burn_drive_hal_get_media_type_full (NautilusBurnDrive *drive,
 	}
 
  fallback:
+#ifdef ENABLE_NON_HAL_FALLBACK_CODE
 	return nautilus_burn_drive_get_media_type_from_path_full (drive->device,
 								  is_rewritable,
 								  is_blank,
 								  has_data,
 								  has_audio);
+#else
+	return NAUTILUS_BURN_MEDIA_TYPE_ERROR;
+#endif
 }
 #endif
 
@@ -1217,6 +1207,7 @@ nautilus_burn_drive_get_media_size_from_path (const char *device)
 	return size;
 }
 
+#ifdef ENABLE_NON_HAL_FALLBACK_CODE
 static gint64
 try_media_size_from_type (NautilusBurnDrive *drive)
 {
@@ -1287,6 +1278,7 @@ try_media_size_from_type (NautilusBurnDrive *drive)
 
 	return size;
 }
+#endif
 
 #ifdef USE_HAL
 static gint64
@@ -1359,12 +1351,16 @@ nautilus_burn_drive_get_media_size (NautilusBurnDrive *drive)
 
 #ifdef USE_HAL
 	size = nautilus_burn_drive_hal_get_media_size (drive);
+
+#ifdef ENABLE_NON_HAL_FALLBACK_CODE
 	if (size < 0) {
 		size = nautilus_burn_drive_get_media_size_from_path (drive->device);
 		if (size < 0) {
 			size = try_media_size_from_type (drive);
 		}
 	}
+#endif
+
 #else
 	size = nautilus_burn_drive_get_media_size_from_path (drive->device);
 #endif
@@ -1504,6 +1500,7 @@ unmount_timeout (gpointer data)
 	return FALSE;
 }
 
+#ifndef USE_GNOME_MOUNT
 static const char *umount_known_locations [] = {
 	"/sbin/umount",
 	"/bin/umount",
@@ -1511,28 +1508,28 @@ static const char *umount_known_locations [] = {
 	"/usr/bin/umount",
 	NULL
 };
+#endif
 
 /* Returns the full command */
 static GPtrArray *
 create_command (const char *device)
 {
 	GPtrArray *argv;
+	char      *str;
 
 	argv = g_ptr_array_new ();
 
-	if (use_gnome_mount ()) {
-		char *str;
-
-		str = g_strdup ("gnome-mount");
-		g_ptr_array_add (argv, str);
-		str = g_strdup_printf ("--device=%s", device);
-		g_ptr_array_add (argv, str);
-		str = g_strdup ("--unmount");
-		g_ptr_array_add (argv, str);
-		str = g_strdup ("--no-ui");
-		g_ptr_array_add (argv, str);
-	} else {
-		char *str;
+#ifdef USE_GNOME_MOUNT
+	str = g_strdup (BINDIR "/gnome-mount");
+	g_ptr_array_add (argv, str);
+	str = g_strdup_printf ("--device=%s", device);
+	g_ptr_array_add (argv, str);
+	str = g_strdup ("--unmount");
+	g_ptr_array_add (argv, str);
+	str = g_strdup ("--no-ui");
+	g_ptr_array_add (argv, str);
+#else
+	{
 		int   i;
 
 		str = NULL;
@@ -1552,6 +1549,9 @@ create_command (const char *device)
 		str = g_strdup_printf ("%s", device);
 		g_ptr_array_add (argv, str);
 	}
+#endif
+
+	g_ptr_array_add (argv, NULL);
 
 	return argv;
 }
@@ -1559,6 +1559,7 @@ create_command (const char *device)
 static void *
 unmount_thread_start (void *arg)
 {
+	GError      *error;
 	UnmountData *data;
 	gint         exit_status;
 	char        *envp [] = {
@@ -1569,7 +1570,8 @@ unmount_thread_start (void *arg)
 	data = arg;
 
 	data->unmount_ok = TRUE;
-	
+
+	error = NULL;
 	if (g_spawn_sync (NULL,
 			  (char **)data->argv->pdata,
 			  envp,
@@ -1578,7 +1580,7 @@ unmount_thread_start (void *arg)
 			  NULL,
 			  NULL,
 			  &exit_status,
-			  NULL)) {
+			  &error)) {
 		if (exit_status == 0) {
 			data->unmount_ok = TRUE;
 		} else {
@@ -1589,6 +1591,10 @@ unmount_thread_start (void *arg)
 		sleep (2);
 	} else {
 		/* spawn failure */
+		if (error) {
+			g_warning ("Unable to unmount: %s", error->message);
+			g_error_free (error);
+		}
 		data->unmount_ok = FALSE;
 	}
 
@@ -1649,6 +1655,93 @@ nautilus_burn_drive_unmount (NautilusBurnDrive *drive)
 	return unmount_ok;
 }
 
+#ifdef USE_HAL
+static gboolean
+nautilus_burn_drive_hal_is_mounted (NautilusBurnDrive *drive)
+{
+	gboolean       is_mounted;
+	LibHalContext *ctx;
+	char          *child_udi;
+	DBusError      error;
+
+	g_return_val_if_fail (drive != NULL, FALSE);
+
+	is_mounted = FALSE;
+
+	if (drive->priv->udi == NULL) {
+		goto done;
+	}
+		
+	ctx = drive->priv->ctx;
+
+	if (ctx == NULL) {
+		goto done;
+	}
+
+	child_udi = hal_drive_get_first_child_udi (drive);
+	if (child_udi == NULL) {
+		goto done;
+	}
+
+	dbus_error_init (&error);
+	is_mounted = libhal_device_get_property_bool (ctx, 
+						      child_udi,
+						      "volume.is_mounted",
+						      &error);
+	if (dbus_error_is_set (&error)) {
+		g_warning ("%s\n", error.message);
+		dbus_error_free (&error);
+	}
+
+	g_free (child_udi);
+ done:
+	return is_mounted;
+}
+#else
+static gboolean
+nautilus_burn_drive_nohal_is_mounted (NautilusBurnDrive *drive)
+{
+	gboolean              is_mounted;
+	gint64                media_size;
+	NautilusBurnMediaType type;
+
+	media_size = nautilus_burn_drive_get_media_size (drive);
+	type = nautilus_burn_drive_get_media_type (drive);
+
+	is_mounted = (type == NAUTILUS_BURN_MEDIA_TYPE_BUSY)
+		|| (media_size == NAUTILUS_BURN_MEDIA_SIZE_BUSY);
+
+	return is_mounted;
+}
+#endif
+
+/**
+ * nautilus_burn_drive_is_mounted:
+ * @drive: #NautilusBurnDrive
+ *
+ * Determine if media in the specified drive is mounted.
+ *
+ * Return value: %TRUE if the media is mounted, %FALSE otherwise.
+ *
+ * Since: 2.14
+ **/
+gboolean
+nautilus_burn_drive_is_mounted (NautilusBurnDrive *drive)
+{
+	gboolean is_mounted;
+
+	g_return_val_if_fail (drive != NULL, FALSE);
+
+	is_mounted = FALSE;
+#ifdef USE_HAL
+	is_mounted = nautilus_burn_drive_hal_is_mounted (drive);
+#else
+	is_mounted = nautilus_burn_drive_nohal_is_mounted (drive);
+#endif
+
+	return is_mounted;
+}
+
 /**
  * nautilus_burn_drive_eject:
  * @drive: #NautilusBurnDrive
@@ -1667,14 +1760,15 @@ nautilus_burn_drive_eject (NautilusBurnDrive *drive)
 
 	g_return_val_if_fail (drive != NULL, FALSE);
 
-	if (drive->device == NULL)
+	if (drive->device == NULL) {
 		return FALSE;
-	
-	if (use_gnome_mount ()) {
-		cmd = g_strdup_printf ("gnome-mount --eject --no-ui --device=%s", drive->device);
-	} else {
-		cmd = g_strdup_printf ("eject %s", drive->device);
 	}
+	
+#ifdef USE_GNOME_MOUNT
+	cmd = g_strdup_printf ("gnome-mount --eject --no-ui --device=%s", drive->device);
+#else
+	cmd = g_strdup_printf ("eject %s", drive->device);
+#endif
 
 	res = g_spawn_command_line_sync (cmd, NULL, NULL, NULL, NULL);
 	g_free (cmd);
@@ -3227,6 +3321,58 @@ nautilus_burn_drive_media_type_get_string (NautilusBurnMediaType type)
 	return _("Broken media type");
 }
 
+/**
+ * nautilus_burn_drive_media_type_is_writable
+ * @type: A #NautilusBurnMediaType
+ * @is_blank: if the media type is blank or not
+ *
+ * Determine if a media type is writable
+ *
+ * Returns: %TRUE if the media type can be written to, otherwise return %FALSE.
+ *
+ * Since: 2.14
+ **/
+gboolean
+nautilus_burn_drive_media_type_is_writable (NautilusBurnMediaType type,
+					    gboolean              is_blank)
+{
+	gboolean can_write;
+
+	can_write = FALSE;
+
+	switch (type) {
+	case NAUTILUS_BURN_MEDIA_TYPE_BUSY:
+	case NAUTILUS_BURN_MEDIA_TYPE_ERROR:
+	case NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN:
+	case NAUTILUS_BURN_MEDIA_TYPE_CD:
+	case NAUTILUS_BURN_MEDIA_TYPE_DVD:
+	case NAUTILUS_BURN_MEDIA_TYPE_DVD_RAM:
+		can_write = FALSE;
+		break;
+	case NAUTILUS_BURN_MEDIA_TYPE_CDR:
+	case NAUTILUS_BURN_MEDIA_TYPE_DVDR:
+	case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R:
+	case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R_DL:
+		if (is_blank) {
+			can_write = TRUE;
+		} else {
+			can_write = FALSE;
+		}
+		break;
+	case NAUTILUS_BURN_MEDIA_TYPE_CDRW:
+	case NAUTILUS_BURN_MEDIA_TYPE_DVDRW:
+	case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW:
+		can_write = TRUE;
+		break;
+	default:
+		g_warning ("Unknown media type: %d", type);
+		can_write = FALSE;
+		break;
+	}
+
+	return can_write;
+}
+
 #ifdef USE_HAL
 static void
 set_monitor_enabled_hal (NautilusBurnDrive *drive,
